%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_calc_E
%     
%     Calculates the energy consumption per hop, based on route
%     configuration vector and channel gain vector
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     b: modulation order vector
%     P_tx: power vector
%     
%     Output:
%     E_vec: Energy vector
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function E_vec = fnc_calc_E(c_sys, c_sim, b, P_tx)

%     Check if b and P_tx vecors of same dimension
    if ((length(b) > 1) && (length(P_tx) > 1))
        if (length(b) ~= length(P_tx))
           disp('ERROR IN calc_E. If b and P_tx vectors, they must be of same length')
           b
           P_tx
           size(b)
           size(P_tx)
           pause
        end
    end
    
    E_vec = zeros(1,length(b));

%     The power effiency for each hop
    alph_vec = fnc_calc_PA_eff_fac(c_sys.eta, b);
%     Transmission time
    K = (c_sys.L_pkt/c_sys.B)*1000;
    T_on = K./b;
%     Energy penalty for transient time
    E_tr = c_sys.P_tr*c_sys.T_tr*length(b);
%     Energy vector
    
%     With power consumption:
    E_vec = ((1+alph_vec).*P_tx.*T_on + (c_sys.P_ctx+c_sys.P_crx).*T_on + E_tr)/1000;

return



