%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_calc_P_tx
%     
%     Calculate the power vector, given vectors g and b
%     g and b must be of the same lengts (scalar or vector), or one can
%     be a scalar
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     b: modulation order vectors
%     g: Link gain vector
%     
%     Output:
%     p: power vector
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function p = fnc_calc_P_tx(c_sys, c_sim, b, g)

    if ((length(b) > 1) && (length(g) > 1))
       if (length(b) ~= length(g))
          disp('ERROR IN fnc_calc_P_tx. If b and p vector, they must be of same length')
          b_length = length(b)
          p_length = length(g)
       end
    end

    snr_min = fnc_calc_snr_min(c_sim.P_b, b, c_sim.pwr_m);
    p = (c_sys.N_tot./g).*snr_min;

return