%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_calc_snr_min
%     
%     Calculates the required received SNIR threshold, given a target
%     BER and a modulation order
%     
%     Input:
%     Pb: Target BER (scalar)
%     b: modulation order vector (or scalar)
%     pwr_m: Extra safety margin
%     
%     Output:
%     gma_min: SNIR theshold vector. Same dimension as b
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function gma_min = fnc_calc_snr_min(Pb, b, pwr_m)

%     Debugging
    if (find(b <= 0))
        disp('b is 0!!!')
    end
    
    gma_min = zeros(1,length(b));
    
    ind_psk = find(b < 2);
    ind_qam = find(b >= 2);
    
    b_psk = b(ind_psk);
    b_qam = b(ind_qam);
    
    gma_psk = log(2./(Pb*b_psk)).*(1./(sin(pi./(2.^b_psk)).^2))*(1+pwr_m/100);    
    gma_qam = log((4*(1-(1./sqrt(2.^b_qam))))./(Pb*b_qam)).*(((2.^b_qam)-1)*(2/3))*(1+pwr_m/100);
    
    gma_min(ind_psk) = gma_psk;
    gma_min(ind_qam) = gma_qam;
    
return
