%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_hop_tx
%     
%     Transmission over a single hop. Compares calculated, received
%     SNIR with required threshold for given modulation order
%     
%     Input:
%     c_sys: system parameters
%     c_sis: simulation parameters
%     g: Link gain
%     b: modulation order
%     P_t: Transmission power
%     
%     Output:
%     res: Result vector, consisting of:
%         flag: 1 if success, -1 if fail
%         b: modulation order
%         P_t: Transmission power
%         T: Transmission time
%         E: Used energy
%         snr_rec: Received SNIR
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = fnc_hop_tx(c_sys, c_sim, g, b, P_t);
% [flag b P T E snr_rec]
% 

% % Debugging
% if(P_t == 1)
%     disp('*** Minimum Tx power used! ***') 
% elseif (P_t < 1)
%     disp('*** Even smaller TX power used ***')    
% end

% if (b > c_sys.b_max)
%     disp('b exeeds b_max in fnc_hop_tx(...)')
%     g
%     b
%     P_t
%     pause
% end

flag = 1;
% Find minimum SNR threshold
snr_min = fnc_calc_snr_min(c_sim.P_b, b, 0); % No extra margin here
snr_rec = P_t*g/(c_sys.N_tot);
T = fnc_calc_T(c_sys, c_sim, b);
E = fnc_calc_E(c_sys, c_sim, b, P_t);

if (snr_rec < snr_min)
    if (abs(snr_rec-snr_min) > 0.0001)
        flag = -1;
% %         Debugging
%         disp(['SNR ERROR IN TRANSMISSION!'])
%         disp(['Received SNR: ' num2str(10*log10(snr_rec)) ' dBm'])
%         disp(['Required SNR: ' num2str(10*log10(snr_min)) ' dBm'])
    end
end

% Return hop results
res = [flag b P_t T E snr_rec];
return