%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_linkad_find_b_max
%     
%     Calculates the maximum modulation order per hop for a given route
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     g_vec: Link gain vector
%     
%     Output:
%     b_max: Maximum modulation order vector
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function b_max = fnc_linkad_find_b_max(c_sys, c_sim, g_vec)

b_max = zeros(1,length(g_vec));

% modulation order vector
b = c_sys.b_min:c_sys.b_stp:c_sys.b_max;

% SNR limits
snr_lim_vec = fnc_calc_snr_min(c_sim.P_b, b, c_sim.pwr_m);
% Find required transmission power
P_tx_max_vec = fnc_calc_P_tx_max(c_sys, b);

for n = 1:length(g_vec)
    
%     Largest received SNR for given hop
    snr_r_max = (P_tx_max_vec*g_vec(n))/c_sys.N_tot;
%     Find valid
    inx = max(find(snr_r_max > snr_lim_vec));
    
%     Select largest, if exists
    if (length(inx) > 0)
        b_max(n) = c_sys.b_min + (inx-1)*c_sys.b_stp;
    else
        b_max(n) = -1;
    end
    
end

return