%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_nonad_opt
%     
%     Channel inversion, fixed rate route configuration.
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     ch_gains: Link gain vector
%     
%     Output:
%     res: result vector of dimension Lx2, consists of
%         b: Modulation order, -1 if no valid config exists
%         P_tx: transmission power
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = fnc_nonad_opt(c_sys, c_sim, ch_gains)

res = zeros(c_sim.L_route, 2);

% Time Slots
% T_hop = (c_sim.T - c_sys.T_tr*c_sim.L_route)/c_sim.L_route;

% Transmission power
b_vec = c_sim.b_fix*ones(1,c_sim.L_route);
p_tx_vec = fnc_calc_P_tx(c_sys, c_sim, b_vec, ch_gains);

b_ul = fnc_linkad_find_b_max(c_sys, c_sim, ch_gains);
T_used = sum(fnc_calc_T(c_sys, c_sim, b_vec));

br_ind = find(b_vec > b_ul);

if (length(br_ind) > 0)
    
%     disp('b_ul < b_ll in fnc_nonad_opt(...)')
%     b_vec
%     b_ul

    b_vec(br_ind) = -1;
    p_tx_vec = fnc_calc_P_tx(c_sys, c_sim, b_ul, ch_gains);
    
    res = [b_vec' p_tx_vec'];
    return
    
elseif (T_used > c_sim.T)
    
%     disp('T_min > T_lim in fnc_nonad_opt_d(...)')
%     T_used
%     c_sim.T

    res = [-1*ones(c_sim.L_route,1) zeros(c_sim.L_route,1)];
    return

end

res = [b_vec' p_tx_vec'];

return