%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   fnc_route_calc_res
%     
%     Calculates the averages of a route transmission
%     
%     Input:
%     c_sim: system parameters
%     hop_res: Matrix of hop results. L rows
%     conf_succ: Boolean value if the configuration was successful.
%     
%     Output:
%     res: Route averages, consisting of:
%         flag: status flag. 1: OK,  -1: setup err, -2: Tx err
%         E: Total energy consumption
%         T: Total end-to-end delay
%         b_avg: average modulation order
%         P_avg: average transmission power
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = fnc_route_calc_res(c_sim, hop_res, conf_succ)
% route_res: [flag b P T E SNR]
% acc_res: [flag E T b_avg P_avg] 1: OK,  -1: setup err, -2: Tx err

res = zeros(1,5);

if (conf_succ)
    %     Energy consumed also if failed transmission
    res(2) = sum(hop_res(:,5)); % acc energy
    if (length(find(hop_res(:,1) > 0)) == c_sim.L_route)
        res(1) = 1; %OK
        res(3) = sum(hop_res(:,4));  % end-to-end time, if succ
        res(4) = mean(hop_res(:,2)); % avg b, if succ
        res(5) = mean(hop_res(:,3)); % avg P, if succ        
    else
        res(1) = -2;   %TX error, energy is still used
    end
else
    res(1) = -1; %Setup error
end

