%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   nt_fnc_find_M_closest
%     
%     Finds the average of the M closest nodes that are closer to another,
%     given node s than node n itself
%     
%     Input:
%     nw: Coordingates of all nodes in network
%     n: Index of node in question
%     s: Index of sink node
%     M: number of nodes to be found
%     
%     Output:
%     M_mean: The mean of the distances from n to the M closes nodes, in
%     the direction of s.
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function M_mean = nt_fnc_find_M_closest(nw, n, s, M)

N = length(nw);
succ = 0;

% Large initial value, for comparison
% Store the M closest nodes, in direction of center
init_max = 10*max(nw(1,:));
dst_M_closest = 1000;

% The node in question, origin
node = nw(:,n);
% Center node, sink
sink = nw(:,s);

% Distance from origin to sink
d_o_s = nt_fnc_node_dist(node,sink);

for m=1:N
%     Distance from origin to node m
    d_o_i = nt_fnc_node_dist(node,nw(:,m));
%     Distance from other node to sink
    d_i_s = nt_fnc_node_dist(nw(:,m),sink);
    
%     Find the M closest nodes
%     check that considered node is not sink, or closer 
%     to the sink than the other
    if ((d_i_s < d_o_s) && (m ~= s))
        
%         Check if node m is among the M closest nodes by by comparing 
%         with the one furthest away (dst_shortest is sorted) and that node
%         is closer to considered than to sink
        if((d_o_i < dst_M_closest(end)) && (d_o_s > d_o_i))
%             If so, add to list of shortest
            dst_M_closest(end) = d_o_i;
%             Sort the list
            dst_M_closest = sort(dst_M_closest);
            succ = succ + 1;
        end
    end
    
end

% If no closer nodes found, the node is one if the M+1 nodes closest 
% to the sink. Disregard this result and return negative values. Otherwise,
% return the min, max and avg of the list
if (succ < M)
    M_mean = -1;
else
    M_mean = mean(dst_M_closest);
end

return
