%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   nt_fnc_next_node
%     
%     Finds the next, closest node in direction of given sink. Part of 
%     shortest-hop routing algorithm.
%     
%     Input:
%     nw: Coordingates of all nodes in network
%     node_ind: Index of node in question
%     sink_ind: Index of sink node
%     
%     Output:
%     dst: Distance to the next node
%     ind: Index of next node
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [dst, ind] = nt_fnc_next_node(nw,node_ind, sink_ind)

    N = length(nw);
    dst = 1000;
    ind = 0;
    node = nw(:,node_ind);
    sink = nw(:,sink_ind);
    
    %find dst to sink
    d_o_s = nt_fnc_node_dist(node,sink); 
    for m=1:N %run through all other nodes
        
        %find dst between other node and sink
        d_i_s = nt_fnc_node_dist(nw(:,m),sink); 
        %if node is closer to sink
        if (d_i_s < d_o_s) 
            %find dist between node and other node
            d_o_i = nt_fnc_node_dist(node,nw(:,m)); 
            %check if closer then best, and not self
            if (d_o_i < dst  && d_o_i > 0) 
                %Distance between node and next node
                dst = d_o_i;
                ind = m;
            end
        end
    end

%if no closer node found, return distance to origin, and 0 as next
if (ind == 0)
   dst = d_o_s;
   ind = sink_ind;
end
    
return