%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%     function:   nt_fnc_show_route
%     
%     Calculates Route statistics. Visualizes route if desired.
%     
%     Input:
%     nw: Coordingates of all nodes in network
%     nodes: Index of nodes in route
%     x: Side of square network area
%     fig_n: Boolean value. Show route in figure if 1.
%     
%     Output:
%     res: Route stats, consists of
%         N_hop: Number of hops from source to sink
%         h_min: Shortest hop
%         h_max: Longest hop
%         h_avg: Average hop
%         d_shortest: Euxledian distance between sink and source
%         d_route_tot: Total route length
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function res = nt_fnc_show_route(nw, nodes, dst, x, fig_n)
% [N_hop, h_min, h_max, h_avg, d_shortest, d_route_tot]

N_hop = length(nodes)-1;

if (fig_n ~= 0)

    fig_1 = figure(fig_n);
    set(fig_1,'Name','Network sketch');
    scatter(nw(1,:)',nw(2,:)',40,'filled');
    axis([-x/2 x/2 -x/2 x/2])
    grid on
    xlabel('x-coordinate [m]')
    ylabel('y-coordinate [m]')
    hold on
    scatter(nw(1,nodes(1)),nw(2,nodes(1)),40,'g','filled');
    scatter(nw(1,nodes(end)),nw(2,nodes(end)),40,'r','filled');
    
end

for (m=1:N_hop)
    
    n1 = nw(:,nodes(m));
    n2 = nw(:,nodes(m+1)); 
    
    if (n1(1) > n2(1))
        x_neg = 1;
    end
    
    if (n1(2) > n2(2))
        y_neg = 1;
    end
    
    x_diff = n2(1)-n1(1);
    y_diff = n2(2)-n1(2);

    x_step = x_diff/2;
    y_step = y_diff/2;
    
    if (fig_n ~= 0)
        plot(n1(1):x_step:n2(1),n1(2):y_step:n2(2),'r')
    end
    
end

d_min = min(dst);
d_max   = max(dst);
d_avg   = mean(dst);

% Shortest discance between source and sink
dst_shortest  = nt_fnc_node_dist(nw(:,nodes(1)),nw(:,nodes(end)));
% Total route length
dst_route = sum(dst);

% [N_hop, h_min, h_max, h_avg, d_shortest, d_route_tot]
res = [N_hop d_min d_max d_avg dst_shortest dst_route];

return