%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     function:   print_results
%     
%     Presents the detailed route results in table form
%     
%     Input:
%     c_sys: system parameters
%     c_sim: simulation parameters
%     res: Result matrix, number of rows equals numbers of hops in route
%     
%     Output:
%     none
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function print_results(c_sys, c_sim, res)
% res: [flag b P T E SNR]

disp(['|-------|--------|-------|------- --------|---------|-----------|'])
disp(['|  Hop  |  flag  |   b   |  Ptx  |  T_on  |E/bit[uJ]|  SNR[dB]  |']) 
disp(['|-------|--------|-------|------- --------|---------|-----------|'])
for m=1:c_sim.L_route
    if (res(m,6) > 0)
        snr_db = 10*log10(res(m,6));
    else
        snr_db = 0;
    end
    [s_res, e_res] = sprintf('|%-7.0f|%-8.0f|%-7.2f|%-7.2f|%-8.2f|%-9.2f|%-11.2f|', m,res(m,1:4),1000*res(m,5)/c_sys.L_pkt, snr_db);
    disp(s_res)    
end
disp(['|-------|--------|-------|------- --------|---------|-----------|'])

if (length(find(res(:,1) == 1)) == c_sim.L_route)
    disp('***** Transmission OK *****')
    E_total     = sum(res(:,5));
    E_per_bit   = E_total/c_sys.L_pkt;
    E_per_bit_dBm = 10*log10(E_per_bit);
    T_total     = sum(res(:,4));
    T_mean      = mean(res(:,4));
    E_per_bit_per_hop = mean(res(:,5))/c_sys.L_pkt;
    E_per_bit_per_hop_dB = 10*log10(E_per_bit_per_hop);
    
    disp([' Total energy consumtion: ' num2str(E_total) ' mJ'])
    disp([' Total end-to-end time  : ' num2str(T_total) ' ms'])
    disp([' Average T_on per hop: ' num2str(T_mean) ' ms'])
else
    disp('***** Transmission ERROR *****')
end

return