clear all
close all
clc

x_size  = 200; %Area size
N_nd = 100; % # Nodes
N_cl = 3; % Find averages of N_cl closest for each

N_runs = 100;

res = zeros(N_nd*N_runs,1);
res_route = zeros(N_runs,6);

% big counter
for bc=1:N_runs

    clear nodes
    clear dst
    
    disp([ num2str(bc) '/' num2str(N_runs) ])

    % Create network
    nw = nt_fnc_create_nw(x_size,N_nd);
    
%     Find random sink
    sink = ceil(rand*N_nd);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Find average of N closest node, which closer to sink than itself, for
    % all nodes
    
    % Find min max and avg of N_cl nodes, in direction of center
    
    for n=1:N_nd
        if (n ~= sink)
            res((bc-1)*N_nd+n) = nt_fnc_find_M_closest(nw,n,sink,N_cl);
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Find a route from a random node to the sink, using shortest-hop
    % algorithm

    % Choose a random node
    nodes(1) = ceil(rand*N_nd);
    
    n = 1;
    go = 1;
    
    % While sink not reached
    while (go)

%         Find next node towards sink
        [hop,node_next] = nt_fnc_next_node(nw,nodes(n),sink);
        % Vector of distances        
        dst(n) = hop;
        % Vector of node indices
        nodes(n+1)= node_next;
        if (node_next == sink)
            % Sink found
            go = 0;
        end
        n = n + 1;
    end

    if (bc == N_runs)
        fig_route = 1;
    else
        fig_route = 0;
    end

    %Calc route stats
    res_route(bc,:) = nt_fnc_show_route(nw, nodes, dst, x_size, fig_route);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract stats

    % Remove the nodes closest to the sink (returned negative values)
    indxs = find(res > 0);
    res_mod = res(indxs);
    
    % Extract the mean and variance of the results    
    res_avg = mean(res_mod);
    res_var = var(res_mod);
    res_min = min(res_mod);
    res_max = max(res_mod);
    
    n_bins = 50;
    [res_bin, res_ind] = hist(res_mod,n_bins);
%     res_nrm = sum(res_bin);
    [res_bin_max, res_bin_max_ind] = max(res_bin);
    res_bin_norm = res_bin./max(res_bin);
    res_peak = res_ind(res_bin_max_ind);
    
    % Extract stats from routes
    % [N_hop, h_min, h_max, h_avg, d_shortest, d_route_tot]
    avg_N_hop = mean(res_route(:,1));
    d_hop_min = min(res_route(:,2));
    d_hop_max = max(res_route(:,3));
    d_hop_avg = mean(res_route(:,4));
    d_straight_avg = mean(res_route(:,5));
    d_route_avg = mean(res_route(:,6));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Show results

fig_hist    = 2;
fig_pdf_rl  = 3;
fig_cdf_rl  = 4;

% Some key statistics
disp('Network statistics:')
disp(['   Size of area___________________: ' num2str(x_size) ' x ' num2str(x_size)])
disp(['   Number of nodes________________: ' num2str(N_nd)])
disp(['   Nodes per square meter_________: ' num2str(N_nd/x_size^2) ])

disp(' ')
disp(['For every node, found the mean of distance to ' num2str(N_cl)]);
disp(['closest nodes that are closer to sink than node itself: '])
disp(['   Stats from this:'])
disp(['      Mean (of means)_____________: ' num2str(res_avg) ])
disp(['      Variance (of means)_________: ' num2str(res_var) ])
disp(['      Smallest ___________________: ' num2str(res_min) ])
disp(['      Largest ____________________: ' num2str(res_max) ])
disp(['      Peak of histogram __________: ' num2str(res_peak) ])

disp(' ')
disp('Route statistics:')
disp(['   # hops (using shortest hop)____: ' num2str(avg_N_hop) ])
disp(['   Shortest hop___________________: ' num2str(d_hop_min) ])
disp(['   Longest hop____________________: ' num2str(d_hop_max) ])
disp(['   Average hop____________________: ' num2str(d_hop_avg) ])
disp(['   Dst from source to sink_______ : ' num2str(d_straight_avg) ])
disp(['   Total route length_____________: ' num2str(d_route_avg) ])


% Present plots

% Set limits for PDF/CDF x-axis
l_max = res_avg + res_var;
l_stp = l_max/n_bins;
lims = 0:1/100:l_max;
% Rayleigh PDF and CDF

pdf_rayl_pk = raylpdf(lims,res_peak);
pdf_chi2_pk = chi2pdf(lims,res_peak);
pdf_rayl_mn = raylpdf(lims,res_avg);
pdf_chi2_mn = chi2pdf(lims,res_avg);

cdf_rayl_pk = raylcdf(lims,res_peak);
cdf_rayl_mn = raylcdf(lims,res_avg);
cdf_chi2_pk = chi2cdf(lims,res_peak);
cdf_chi2_mn = chi2cdf(lims,res_avg);




fig = figure(fig_hist);
set(fig,'Name','Statistics of distance to M closest nodes, towards sink');
hold on
norm_fac = max(pdf_chi2_mn);
bar(res_ind, res_bin_norm*norm_fac)
% plot(lims,pdf_rayl_pk,'r');
% plot(lims,pdf_chi2_pk,'g');
% plot(lims,pdf_rayl_mn/sum(pdf_rayl_mn),'g');
% plot(lims,pdf_chi2_mn/sum(pdf_chi2_mn),'r');
% plot(lims,pdf_rayl_mn,'g');
plot(lims,pdf_chi2_mn,'r');
title('Chi-square, mean')

figure
hold on
norm_fac = max(pdf_rayl_mn);
bar(res_ind, res_bin_norm*norm_fac)
plot(lims,pdf_rayl_mn,'g');
title('rayleigh, mean')

figure
hold on
norm_fac = max(pdf_chi2_pk);
bar(res_ind, res_bin_norm*norm_fac)
plot(lims,pdf_chi2_pk,'g');
title('Chi-square, peak')

figure
hold on
norm_fac = max(pdf_rayl_pk);
bar(res_ind, res_bin_norm*norm_fac)
plot(lims,pdf_rayl_pk,'g');
title('rayleigh, peak')

% xlabel('Hop length [m]')
% ylabel('Relative Occurences/Probability')
% legend('Histogram','Rayleigh','Chi-squared');

% fig = figure(fig_cdf_rl);
% set(fig,'Name','CDF');
% hold on
% % plot(lims,cdf_rayl_pk,'r');
% % plot(lims,cdf_chi2_pk,'g');
% plot(lims,cdf_rayl_mn,'g');
% plot(lims,cdf_chi2_mn,'r');
% xlabel('Hop length [m]')
% ylabel('Probability')
% legend('Rayleigh','Chi-square');