close all
clear all
clc

load param_sys
load param_sim

 c_sim.P_b = 1.0005e-004
% c_sys.P_c = 100;
% c_sys.P_max = 250;

b_min = 1;
b_max = 10;

d = 0.1:0.1:80;

b_dsc_small = 1:5;
b_dsc = 1:1:b_max;
b = b_min:c_sys.b_stp:b_max;


g = fnc_ch_calc_gain(c_sys.G_1,c_sys.k,c_sys.M_l,d);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% P vs distance for discrete values of b

p_vs_d = zeros(length(b_dsc),length(d));
for n=1:1:length(b_dsc_small)
    p_vs_d(n,:) = fnc_calc_P_tx(c_sys, c_sim, b_dsc_small(n), g);
end
p_tx_max_flat = fnc_calc_P_tx_max(c_sys, b_dsc_small);
p_tx_max_matr = p_tx_max_flat'*ones(1,length(d));


inxs = find(p_vs_d > 100);
p_vs_d(inxs) = 100;

fig_a = figure('Name','P_tx vs distance');
hold on
plot(d,p_vs_d)
plot(d,p_tx_max_matr)
fig_a_ax = gca;
title(fig_a_ax, ['P_{tx} vs distance for discrete values of bit/symbol. BER: ' num2str(c_sim.P_b) ]);
xlabel(fig_a_ax, 'distance [m]')
ylabel(fig_a_ax, 'P_{tx}')
legend(fig_a_ax, num2str(b_dsc_small'),'Location','EastOutside')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% P_tx vs b for discrete values of d

d_dsc = [5 10 20 40];
% d_dsc = 30;
g_dsc = fnc_ch_calc_gain(c_sys.G_1,c_sys.k,c_sys.M_l,d_dsc);
p_vs_b = zeros(length(d_dsc),length(b));
for n=1:1:length(d_dsc)
    p_vs_b(n,:) = fnc_calc_P_tx(c_sys, c_sim, b, g_dsc(n));
end

P_tx_max = fnc_calc_P_tx_max(c_sys, b);

for n=1:1:length(d_dsc)
    inxs = find(p_vs_b(n,:) > P_tx_max);
%     inxs_compl = find(p_vs_b(n,:) <= P_tx_max);
    p_vs_b(n,inxs) = P_tx_max(inxs);
end
% inxs = find(p_vs_b > ones(length(d_dsc),1)*P_tx_max);
% p_vs_b(inxs) = P_tx_max(1);
% b_matr = ones(length(d_dsc),1)*b;

fig_b = figure('Name','P_tx vs bit/symbol');
hold on
plot(b,p_vs_b)
plot(b,P_tx_max,'k')
fig_b_ax = gca;
title(fig_b_ax, ['P_{tx} vs bit/symbol for discrete values of distance. BER: ' num2str(c_sim.P_b) '. Max TX power dotted'])
xlabel(fig_b_ax,'b [bit/symbol]')
ylabel(fig_b_ax,'P_{tx}')
legend(fig_b_ax,num2str(d_dsc'),'Location','EastOutside')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% g vs d

k_vec = [1 2 3 4];
g_matr = zeros(length(k_vec),length(d));
for n=1:length(k_vec)
    g_matr(n,:) = fnc_ch_calc_gain(c_sys.G_1,k_vec(n),c_sys.M_l,d); 
end

fig_c = figure('Name','Channel gain vs d');
plot(d,10*log10(g_matr))
fig_c_ax = gca;
title(fig_c_ax, 'Channel gain vs distance for different values of path loss exponent')
xlabel(fig_c_ax,'distance [m]')
ylabel(fig_c_ax,'gain [dB]')
legend(fig_c_ax,num2str(k_vec'),'Location','EastOutside')

hold on
g_ref = fnc_ch_calc_gain(c_sys.G_1,c_sys.k,c_sys.M_l,d);
plot(d,10*log10(g_ref),'-.')

grid on

% g_tst = fnc_ch_calc_gain(c_sys.G_1,c_sys.k,1000,d);
% plot(d,10*log10(g_tst),':')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% snr limits for MQAM

Pb = [1e-3 1e-4 1e-5 1e-6];
snr_lim = zeros(length(Pb),length(b));

for n=1:length(Pb)
    snr_lim(n,:) = fnc_calc_snr_min(Pb(n), b, c_sim.pwr_m);
end

fig_d = figure('Name','Mimimum SNR vs b');
plot(b,10*log10(snr_lim))
fig_d_ax = gca;
title(fig_d_ax, 'Minimum received SNR for MQAM with b bit/symbol')
xlabel(fig_d_ax,'b [bit/symbol]')
ylabel(fig_d_ax,'SNR [dB]')
legend(fig_d_ax,num2str(Pb'),'Location','EastOutside')
grid on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Max received SNR vs d. SNR limits in same plot

d_small = 5:0.1:75;
g_small = fnc_ch_calc_gain(c_sys.G_1,c_sys.k,c_sys.M_l,d_small);
p_tx_max_small = fnc_calc_P_tx_max(c_sys, b_dsc_small);
snr_rec_max = p_tx_max_small'*g_small/c_sys.N_tot;

snr_lim_dsc = fnc_calc_snr_min(c_sim.P_b, b_dsc_small, c_sim.pwr_m);
snr_lim_matr = snr_lim_dsc'*ones(1,length(d_small));

fig_e = figure('Name','Max received SNR vs d');
hold on
plot(d_small,10*log10(snr_rec_max),'--');
plot(d_small,10*log10(snr_lim_matr));
fig_e_ax = gca;
title(fig_e_ax, ['Max received SNR vs d, compared with required SNR for discrete MAQM. BER: ' num2str(c_sim.P_b) ])
xlabel(fig_e_ax,'distance [m]')
ylabel(fig_e_ax,'SNR [dB]')
legend(fig_e_ax,num2str(b_dsc_small'),'Location','EastOutside')
grid on

c_sys