close all
clear all
clc

load param_sys
load param_sim
c_sim.pwr_m = 0;
c_sim.P_b = 1e-3;

c_sys
c_sim

% Show E and P vs d and b for a single hop


d_min = 1;
d_max = 50;
d_stp = (d_max-d_min)/99;

b_min = 1;
b_max = 10;
b_stp = 0.1;

% Distance and bit/symbol
% d = 10;
d = d_min:d_stp:d_max;
b = b_min:b_stp:b_max;

% Gains
g = fnc_ch_calc_gain(c_sys.G_1, c_sys.k, c_sys.M_l, d);

N_d = length(d);
N_b = length(b);

% matrices for energy/tx power as function of d and b
e_m = zeros(N_b, N_d);
p_m = zeros(N_b, N_d);
% To keep track of successful conigurations
chk = ones(N_b, N_d);

% Time consumption
t_vec = zeros(size(b));

for n=1:N_b
%     for n=1:N_b
        p_m(n,:) = fnc_calc_P_tx(c_sys, c_sim, b(n), g);
        p_tx_max = fnc_calc_P_tx_max(c_sys, b(n));
        %Check if power constraint breached
        p_tx_br_ind = find(p_m(n,:) > p_tx_max);
%         p_m(n,p_tx_br_ind) = p_tx_max;
        p_m(n,p_tx_br_ind) = 0;
%         p_m(n,p_tx_br_ind) = (c_sys.P_max - c_sys.P_c);
        e_m(n,:) = fnc_calc_E(c_sys, c_sim, b(n), p_m(n,:));
        e_m(n,p_tx_br_ind) = 0;

%       Check received and required SNR  
        snr_min = fnc_calc_snr_min(c_sim.P_b, b(n), c_sim.pwr_m);
        snr_rec = p_m(n,:).*g/c_sys.N_tot;
        snr_br_ind = find(snr_rec < snr_min);
%         If unsuccesful, set value to -1
        chk(n,snr_br_ind) = -1;
        
%     end
end



fig_a = figure('Name','Minimum P_tx vs distance distance and bit/symbol');
surf(d,b,p_m)
fig_a_ax = gca;
title(fig_a_ax, ['Minimum P_{tx} vs distance distance and bit/symbol. BER: ' num2str(c_sim.P_b) ]);
ylabel(fig_a_ax, 'b [bit/symbol]')
xlabel(fig_a_ax, 'distance [m]')
zlabel(fig_a_ax, 'P_{tx} [mW]')

% Show figures
fig_b = figure('Name','E_pkt vs distance distance and bit/symbol');
surf(d,b,(e_m/c_sys.L_pkt))
fig_b_ax = gca;
title(fig_b_ax, ['E_{pkt} vs distance distance and bit/symbol. BER: ' num2str(c_sim.P_b) ]);
ylabel(fig_b_ax, 'b [bit/symbol]')
xlabel(fig_b_ax, 'distance [m]')
zlabel(fig_b_ax, 'Energy/bit [dBmJ]')


d_fix = 11.5;
b_trunc = 1:0.001:8;
c_sys.P_ctx = 0;
c_sys.P_crx = 0;
c_sys.P_tr = 0;

g_fix = fnc_ch_calc_gain(c_sys.G_1, c_sys.k, c_sys.M_l, d_fix);
P_tx_fix_d = fnc_calc_P_tx(c_sys, c_sim, b_trunc, g_fix);
P_tx_max_vs_b = fnc_calc_P_tx_max(c_sys, b_trunc);
fix_ind_br = find(P_tx_fix_d > P_tx_max_vs_b);
E_fix_d = fnc_calc_E(c_sys, c_sim, b_trunc, P_tx_fix_d);

E_fix_d(fix_ind_br) = 0;
P_tx_fix_d(fix_ind_br) = 0;

E_fix_d_db = 10*log10(E_fix_d/c_sys.L_pkt);

fig_c = figure('Name','P_tx and E vs b for fixed distance');
hold on
[fig_c_ax,h1,h2] = plotyy(b_trunc,P_tx_fix_d,b_trunc,E_fix_d_db);

set(get(fig_c_ax(1),'Ylabel'),'String','P_{tx} [mW]')
set(get(fig_c_ax(2),'Ylabel'),'String','Energy/bit [dBmJ]')
xlabel(fig_c_ax(1), 'b [bit/symbol]')
title(['P_{tx} and total energy for fixed distance d = ' num2str(d_fix) ' (g = ' num2str(10*log10(g_fix)) ' dB)'])
% legend(h1, 'Required transmission power')
% legend(fig_c_ax(2), 'Energy consumption')


