%% INITIALIZATION
clear all,close all,clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% System parameters (Adjustable)

        % Route length
         L_route = 5;
        %  %Time constraint, end-to-end [ms]
         T = 515;
        % Fixed-rate bit/symbol
         b_fix = 2;
        % Mean hop length [m]
         ch_m = 11.5;
        % Channel gain misadjustment
         g_misad = 1;
        % SNR power margin
         pwr_m = 3;
        % Target BER, from table 1 in (4)
         P_b_end_to_end = 1e-3;
         P_b = 1 - (1 - P_b_end_to_end)^(1/L_route);
%         P_b = 1e-4;
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
        
        c_sim = struct('L_route', L_route, 'T', T, 'b_fix', b_fix, 'ch_m', ch_m, 'P_b_EE', P_b_end_to_end, 'P_b', P_b, 'g_misad', g_misad, 'pwr_m', pwr_m);
        save param_sim c_sim
        scr_constants;
        clear all
        
        load param_sim
        load param_sys
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   No. simulation runs  
    
     N_run = 5000;
     
%     Other variables (temp stuff)
    N_bad_routes = 0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Some variables  
    acc_res_fr = zeros(N_run, 5);
    acc_res_fr_fp = zeros(N_run, 5);
    acc_res_cr = zeros(N_run, 5);
    acc_res_dr = zeros(N_run, 5);
%     acc_res_cr_tsa = zeros(N_run, 5);
%     acc_res_dr_tsa = zeros(N_run, 5);
    
    route_cnf_fr = zeros(N_run, 1);
    route_cnf_cr = zeros(N_run, 1);
    route_cnf_dr = zeros(N_run, 1);    
%     route_cnf_cr_tsa = zeros(N_run, 1);
%     route_cnf_dr_tsa = zeros(N_run, 1);
    
    T_cr = 0;
    T_dr = 0;
    T_fr = 0;
%     T_dr_tsa = 0;
%     T_cr_tsa = 0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%% START GIANT COUNTER

for bigCounter = 1:N_run
    
% %% VARIABLE ROUTE LENGTH
%         % Route length
%          L_route = ceil(rand*10);
%         %  %Time constraint, end-to-end [ms]
%          P_b_end_to_end = 1e-3;
%          P_b = 1 - (1 - P_b_end_to_end)^(1/L_route);
% %          Overwrite
%          c_sim.L_route = L_route;
%          c_sim.P_b = P_b;

%% ROUTE SETUP
    
disp([num2str(bigCounter) '/' num2str(N_run)])
% disp([num2str(bigCounter) '/' num2str(N_run) 'Route Length: ' num2str(L_route)])

% Generate hop lengths
 ch_dist = fnc_ch_create(c_sim.ch_m,c_sim.L_route);
%     ch_dist = [5 12 35];
% ch_dist = [1 2 4 8 10 15 16 20 30 40];
% Find link gains
ch_gains = fnc_ch_calc_gain(c_sys.G_1, c_sys.k, c_sys.M_l, ch_dist);
% Add CSI misadjustment, 1: Gaussian dist, 0: uniform dist
ch_gains_misad = fnc_ch_gain_misad(ch_gains, c_sim.g_misad, 1);
% Check if there exists link that cannot be used
N_bad_routes = N_bad_routes + fnc_route_usable(c_sys, c_sim, ch_gains_misad);

%% OPTIMIZATION

% ### Find the optimal transmission scheme for route ###

% Fixed rate, adaptive power
tic
route_cnf_fr = fnc_nonad_opt(c_sys, c_sim, ch_gains_misad);
T_fr = T_fr + toc;

% Continous rate, dynamic time slot allocation
tic
route_cnf_cr = fnc_linkad_opt_c(c_sys, c_sim, ch_gains_misad);
T_cr = T_cr + toc;

% Discrete rate, dynamic time slot allocation
tic
route_cnf_dr = fnc_linkad_opt_d(c_sys, c_sim, ch_gains_misad);
T_dr = T_dr + toc;


% % % Continous rate, fixed time slot allocation
% tic
% route_cnf_cr_tsa = fnc_linkad_opt(c_sys, c_sim, ch_gains_misad, 0);
% T_cr_tsa = T_cr_tsa + toc;

% % % Discrete rate, fixed time slot allocation
% tic
% route_cnf_dr_tsa = fnc_linkad_opt(c_sys, c_sim, ch_gains_misad, 1);
% T_dr_tsa = T_dr_tsa + toc;


% ### Check if successful transmission scheme found ###

% Fixed rate
if (length(find(route_cnf_fr(:,1) > 0)) == c_sim.L_route)
    route_cnf_fr_ok = 1;
else
    route_cnf_fr_ok = 0; 
end

% Continous rate, dynamic time slot allocation
if (length(find(route_cnf_cr(:,1) > 0)) == c_sim.L_route)
    route_cnf_cr_ok = 1;    
else
    route_cnf_cr_ok = 0;
end

% Discrete rate, dynamic time slot allocation
if (length(find(route_cnf_dr(:,1) > 0)) == c_sim.L_route)
    route_cnf_dr_ok = 1;    
else
    route_cnf_dr_ok = 0;
end

% % Continous rate, fixed time slot allocation
% if (length(find(route_cnf_cr_tsa(:,1) > 0)) == c_sim.L_route)
%     route_cnf_cr_tsa_ok = 1;
% else
%     route_cnf_cr_tsa_ok = 0;
% end

% % Discrete rate, fixed time slot allocation
% if (length(find(route_cnf_dr_tsa(:,1) > 0)) == c_sim.L_route)
%     route_cnf_dr_tsa_ok = 1;    
% else
%     route_cnf_dr_tsa_ok = 0;
% end

%% TRANSMISSION

% ### Transmit only if successful configuration found ###

% Fixed rate
if (route_cnf_fr_ok)
    route_res_fr = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_fr, 0);
    route_res_fr_fp = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_fr, 1);
else
    route_res_fr = zeros(1,6);
    route_res_fr_fp = zeros(1,6);    
end

% Continous rate
if (route_cnf_cr_ok)
    route_res_cr = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_cr, 0);
else
    route_res_cr = zeros(1,6);
end

% Discrete rate
if (route_cnf_dr_ok)
    route_res_dr = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_dr, 0);
else
    route_res_dr = zeros(1,6);
end

% % ### Fixed time slot allocation ###
% % Discrete rate
% if (route_cnf_cr_tsa_ok)
%     route_res_cr_tsa = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_cr_tsa, 0);
% else
%     route_res_cr_tsa = zeros(1,6);
% end
% 
% if (route_cnf_dr_tsa_ok)
%     route_res_dr_tsa = fnc_route_tx(c_sys, c_sim, ch_gains, route_cnf_dr_tsa, 0);
% else
%     route_res_dr_tsa = zeros(1,6);
% end

%% EVALUATION

% Evaluate results
acc_res_fr_fp(bigCounter,:)  = fnc_route_calc_res(c_sim, route_res_fr_fp, route_cnf_fr_ok);
acc_res_fr(bigCounter,:) = fnc_route_calc_res(c_sim, route_res_fr, route_cnf_fr_ok);
acc_res_cr(bigCounter,:) = fnc_route_calc_res(c_sim, route_res_cr, route_cnf_cr_ok);
acc_res_dr(bigCounter,:) = fnc_route_calc_res(c_sim, route_res_dr, route_cnf_dr_ok);
% acc_res_dr_tsa(bigCounter,:) = fnc_route_calc_res(c_sim, route_res_dr_tsa, route_cnf_dr_tsa_ok);
% acc_res_cr_tsa(bigCounter,:) = fnc_route_calc_res(c_sim, route_res_cr_tsa, route_cnf_cr_tsa_ok);

%% END GIANT LOOP
end

%% PRESENTATION

% Simulation parameters
disp(' ')
disp(['# Runs________________________: ' num2str(N_run)])
disp(['# Hops in Route_______________: ' num2str(c_sim.L_route)])
disp(['End-to-end delay constraint___: ' num2str(c_sim.T) ' ms'])
disp(['End-to-end target BER_________: ' num2str(c_sim.P_b_EE)])
disp(['Link gain misadjustment_______: ' num2str(c_sim.g_misad) ' %'])
disp(['Required SNR margin___________: ' num2str(c_sim.pwr_m) ' %'])
disp(['Fixed-rate bit/symbol_________: ' num2str(c_sim.b_fix) ])

% If single run, show details
if (N_run == 1)
    
    % Show SNR limits
    fnc_show_snr_limits(c_sys, c_sim);
    
    %   Present Route information
        disp(' ')
        disp('*** Route Information ***')
    print_route_info(ch_dist, ch_gains, ch_gains_misad);
    disp(' ')
    disp('##########################################################')
    disp('Results for transmission over a single route')
    
%     Transmission results
    if(route_cnf_fr_ok)
        disp(' ')
        disp('*** Fixed Rate, Fixed Power ***')
        print_results(c_sys, c_sim, route_res_fr_fp);
        disp(' ')
        disp('*** Fixed Rate, Variable Power ***')
        print_results(c_sys, c_sim, route_res_fr);
    else
        disp(' ')
        disp('*** Setup Error for Fixed Rate ****')
        print_setup(c_sys, c_sim, route_cnf_fr);
    end
    
    if(route_cnf_cr_ok)
        disp(' ')
        disp('*** Continous Rate, Dynamic time slot allocation ***')
        print_results(c_sys, c_sim, route_res_cr);
    else
        disp(' ')
        disp('*** Setup Error for Continous Rate, Dynamic time slot allocation ****')
        print_setup(c_sys, c_sim, route_cnf_cr);
    end
    
    if(route_cnf_dr_ok)
        disp(' ')
        disp('*** Discrete Rate, Dynamic time slot allocation ***')
        print_results(c_sys, c_sim, route_res_dr);
    else
        disp(' ')
        disp('*** Setup Error for Discrete Rate, Dynamic time slot allocation ****')
        print_setup(c_sys, c_sim, route_cnf_dr);
    end
    
% % %     Fixed time slot allcoation
%     if(route_cnf_cr_tsa_ok)
%         disp(' ')
%         disp('*** Continous Rate, Fixed time slot allocation ***')
%         print_results(c_sys, c_sim, route_res_cr_tsa);
%     else
%         disp(' ')
%         disp('*** Setup Error for Continous Rate, Fixed time slot allocation ****')
%         print_setup(c_sys, c_sim, route_cnf_cr_tsa);
%     end
%     
%     if(route_cnf_dr_tsa_ok)
%         disp(' ')
%         disp('*** Discrete Rate, Fixed time slot allocation ***')
%         print_results(c_sys, c_sim, route_res_dr_tsa);
%     else
%         disp(' ')
%         disp('*** Setup Error for Discrete Rate, Fixed time slot allocation ****')
%         print_setup(c_sys, c_sim, route_cnf_dr_tsa);
%     end
    
end
% Present averaged results
disp(' ')
disp('##########################################################')
disp('Averages')
disp(' ')
disp('# Fixed Rate, Fixed power:')
[E_fr_fp, P_fr_fp, ER_fr_fp] = print_accum_res(c_sys, c_sim, acc_res_fr_fp, N_run);
disp(' ')
disp('# Fixed Rate, Variable power:')
[E_fr, P_fr, ER_fr] = print_accum_res(c_sys, c_sim, acc_res_fr, N_run);
disp(' ')
disp('# Continous Rate, dynamic time slot allocation:');
[E_cr, P_cr, ER_cr] = print_accum_res(c_sys, c_sim, acc_res_cr, N_run);
disp(' ')
disp('# Discrete Rate, dynamic time slot allocation:');
[E_dr, P_dr, ER_dr] = print_accum_res(c_sys, c_sim, acc_res_dr, N_run);
% disp(' ')
% disp('# Continous Rate, fixed time slot allocation:');
% [E_cr_tsa, P_cr_tsa, ER_cr_tsa] = print_accum_res(c_sys, c_sim, acc_res_cr_tsa, N_run);
% disp(' ')
% disp('# Discrete Rate, fixed time slot allocation:');
% [E_dr_tsa, P_dr_tsa, ER_dr_tsa] = print_accum_res(c_sys, c_sim, acc_res_dr_tsa, N_run);

% Compare key results
disp(' ')
disp('##########################################################')
disp('Comparison')

disp(' ')
disp('# Fixed rate, fixed power (FR_FP): ')
disp(['  Energy: ' num2str(10*log10(E_fr_fp)) ' dBm, error rate: ' num2str(ER_fr_fp) ' %'])

disp(' ')
disp('# Fixed rate, variable power (FR_VP): ')
disp(['  Energy: ' num2str(10*log10(E_fr)) ' dBm, error rate: ' num2str(ER_fr) ' %'])
imp = 100*(E_fr_fp - E_fr)/E_fr_fp;
disp(['    Improvement over FR_FP: ' num2str(imp) ' %'])

disp(' ')
disp('# Continous rate, dynamic time (CR): ')
disp(['  Energy: ' num2str(10*log10(E_cr)) ' dBm, error rate: ' num2str(ER_cr) ' %'])
imp = 100*(E_fr_fp - E_cr)/E_fr_fp;
imp2 = 100*(E_fr - E_cr)/E_fr;
disp(['    Improvent over FR_FP: ' num2str(imp) ' %'])
disp(['    Improvent over FR_VP: ' num2str(imp2) ' %'])

disp(' ')
disp('# Discrete rate, dynamic time (DR): ')
disp(['  Energy: ' num2str(10*log10(E_dr)) ' dBm, error rate: ' num2str(ER_dr) ' %'])
imp = 100*(E_dr - E_cr)/E_cr;
disp(['    Degredation from CR: ' num2str(imp) ' %'])

% disp(' ')
% disp('# Continous rate, Fixed time (CR_TSA): ')
% disp(['  Energy: ' num2str(10*log10(E_cr_tsa)) ' dBm, error rate: ' num2str(ER_cr_tsa) ' %'])
% imp = 100*(E_cr_tsa - E_cr)/E_cr;
% disp(['    Degredation from CR: ' num2str(imp) ' %'])

% disp(' ')
% disp('# Discrete rate, fixed time (DR_TSA): ')
% disp(['  Energy: ' num2str(10*log10(E_dr_tsa)) ' dBm, error rate: ' num2str(ER_dr_tsa) ' %'])
% imp = 100*(E_dr_tsa - E_cr_tsa)/E_cr_tsa;
% imp2 = 100*(E_dr_tsa - E_dr)/E_dr;
% disp(['    Degredation from CR_TSA: ' num2str(imp) ' %'])
% disp(['    Degredation from DR: ' num2str(imp2) ' %'])
% 
% disp(' ')
% disp(['Error Rate (should be): ' num2str(100*(N_bad_routes/N_run)) '%'])

% Optimization time
disp(' ')
disp('##########################################################')
disp('Average Time Consumption per iteration')
disp([' #FR_VP: ' num2str(T_fr/N_run)])
disp([' #CR: ' num2str(T_cr/N_run)])
disp([' #DR: ' num2str(T_dr/N_run)])
% disp([' #CR_TSA: ' num2str(T_cr_tsa/N_run)])
% disp([' #DR_TSA: ' num2str(T_dr_tsa/N_run)])


%%